Option Strict On

Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Public Class clsEngine
  Private p_D3DDevice As Device
  Private p_Helper As D3DX
  Private p_target As Control

  Private p_vbCube As VertexBuffer
  Private p_CubeTexture As Texture
  Private p_TransparentCubeTexture As Texture
  Private p_Font As Direct3D.Font

  Private p_LastFPSCheck As Integer
  Private p_FrameCount As Integer
  Private p_FPS As Integer

  Private p_transparent As Boolean = True

  Public p_Init As Boolean = False

  Public Sub New(ByVal target As Control, ByVal width As Integer, ByVal height As Integer, ByVal depth As Integer)
    Dim pp As New PresentParameters()
    With pp
      .BackBufferCount = 1
      .BackBufferWidth = width
      .BackBufferHeight = height
      Select Case depth
      Case 16
        .BackBufferFormat = Format.R5G6B5
      Case 32
        .BackBufferFormat = Format.X8R8G8B8
      End Select

      .SwapEffect = SwapEffect.Discard
      .PresentationInterval = PresentInterval.Immediate

      p_target = target
    End With
    p_Init = InitDevice(pp)
  End Sub

  Public Sub New(ByVal target As Control)
    Dim pp As New PresentParameters()
    With pp
      .Windowed = True

      .SwapEffect = SwapEffect.Discard
      .BackBufferCount = 1
      .BackBufferFormat = Manager.Adapters(0).CurrentDisplayMode().Format
      .BackBufferWidth = CType(target, Form).ClientSize.Width
      .BackBufferHeight = CType(target, Form).ClientSize.Width
      .DeviceWindow = target

      p_target = target
    End With
    p_Init = InitDevice(pp)
  End Sub

  Private Function InitDevice(ByVal pp As PresentParameters) As Boolean
    Dim D3DCaps As Caps = Manager.GetDeviceCaps(0, DeviceType.Hardware)
    Dim CreateFlgs As CreateFlags

    pp.AutoDepthStencilFormat = DepthFormat.D16
    pp.EnableAutoDepthStencil = True

    If D3DCaps.DeviceCaps.SupportsHardwareTransformAndLight() Then
      CreateFlgs = CreateFlags.HardwareVertexProcessing
    ElseIf D3DCaps.DeviceCaps.SupportsPureDevice Then
      CreateFlgs = CreateFlags.PureDevice
    Else
      CreateFlgs = CreateFlags.SoftwareVertexProcessing
    End If
    CreateFlgs = CreateFlgs Or CreateFlags.MultiThreaded

    Try
      p_D3DDevice = New Device(0, DeviceType.Hardware, p_target, CreateFlgs, pp)
    Catch dxerr As DirectXException
      Return False
    Catch
      Return False
    End Try

    p_Font = New Direct3D.Font(p_D3DDevice, New Drawing.Font("Courier New", 13, FontStyle.Bold))
    With p_D3DDevice
      .RenderState.Lighting = False
      .RenderState.ZBufferEnable = True

      .Transform.View = Matrix.LookAtLH(New Vector3(0, 0, -10), _
                                        New Vector3(0, 0, 0), _
                                        New Vector3(0, 1, 0))
      .Transform.Projection = Matrix.PerspectiveFovLH(Math.PI / 4, 4 / 3, 1, 100)
      .Transform.World = Matrix.Identity
    End With
    Return CreateGeometry() AndAlso LoadTextures()
  End Function

  Private Function LoadTextures() As Boolean
    Try
      p_CubeTexture = TextureLoader.FromFile(p_D3DDevice, "Textur.bmp", 512, 512, 1, 0, Format.R5G6B5, Pool.Managed, Filter.Linear, Filter.Linear, 0)
      p_TransparentCubeTexture = TextureLoader.FromFile(p_D3DDevice, "Textur.bmp", 256, 256, 1, 0, Format.R5G6B5, Pool.Managed, Filter.Linear, Filter.Linear, Color.White.ToArgb)
    Catch
      Return False
    End Try

    Return True
  End Function

  Private Function CreateGeometry() As Boolean
    Dim cube() As CustomVertex.PositionTextured
    Try
      p_vbCube = New VertexBuffer(GetType(CustomVertex.PositionTextured), 36, p_D3DDevice, 0, CustomVertex.PositionColored.Format, Pool.Managed)
      cube = DirectCast(p_vbCube.Lock(0, LockFlags.None), CustomVertex.PositionTextured())
    Catch exp As Exception
      Return False
    End Try

    cube(0) = New CustomVertex.PositionTextured(-1, 1, 1, 0, 0)
    cube(1) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)
    cube(2) = New CustomVertex.PositionTextured(-1, 1, -1, 0, 1)
    cube(3) = New CustomVertex.PositionTextured(1, 1, -1, 1, 1)
    cube(4) = New CustomVertex.PositionTextured(-1, 1, -1, 0, 1)
    cube(5) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)

    cube(6) = New CustomVertex.PositionTextured(-1, -1, -1, 1, 0)
    cube(7) = New CustomVertex.PositionTextured(1, -1, 1, 0, 1)
    cube(8) = New CustomVertex.PositionTextured(-1, -1, 1, 0, 0)
    cube(9) = New CustomVertex.PositionTextured(1, -1, 1, 0, 1)
    cube(10) = New CustomVertex.PositionTextured(-1, -1, -1, 1, 0)
    cube(11) = New CustomVertex.PositionTextured(1, -1, -1, 1, 1)

    cube(12) = New CustomVertex.PositionTextured(-1, 1, -1, 0, 0)
    cube(13) = New CustomVertex.PositionTextured(-1, -1, -1, 1, 0)
    cube(14) = New CustomVertex.PositionTextured(-1, 1, 1, 0, 1)
    cube(15) = New CustomVertex.PositionTextured(-1, 1, 1, 0, 1)
    cube(16) = New CustomVertex.PositionTextured(-1, -1, -1, 1, 0)
    cube(17) = New CustomVertex.PositionTextured(-1, -1, 1, 1, 1)

    cube(18) = New CustomVertex.PositionTextured(1, 1, -1, 0, 0)
    cube(19) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)
    cube(20) = New CustomVertex.PositionTextured(1, -1, -1, 0, 1)
    cube(21) = New CustomVertex.PositionTextured(1, -1, 1, 1, 1)
    cube(22) = New CustomVertex.PositionTextured(1, -1, -1, 0, 1)
    cube(23) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)

    cube(24) = New CustomVertex.PositionTextured(-1, 1, -1, 0, 0)
    cube(25) = New CustomVertex.PositionTextured(1, 1, -1, 1, 0)
    cube(26) = New CustomVertex.PositionTextured(-1, -1, -1, 0, 1)
    cube(27) = New CustomVertex.PositionTextured(1, -1, -1, 1, 1)
    cube(28) = New CustomVertex.PositionTextured(-1, -1, -1, 0, 1)
    cube(29) = New CustomVertex.PositionTextured(1, 1, -1, 1, 0)

    cube(30) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)
    cube(31) = New CustomVertex.PositionTextured(-1, 1, 1, 0, 0)
    cube(32) = New CustomVertex.PositionTextured(-1, -1, 1, 0, 1)
    cube(33) = New CustomVertex.PositionTextured(-1, -1, 1, 0, 1)
    cube(34) = New CustomVertex.PositionTextured(1, -1, 1, 1, 1)
    cube(35) = New CustomVertex.PositionTextured(1, 1, 1, 1, 0)

    p_vbCube.Unlock()

    Return True
  End Function

  Public ReadOnly Property Init() As Boolean
  Get
    Return p_Init
  End Get
  End Property

  Public Property Transparent() As Boolean
  Get
    Return p_transparent
  End Get
  Set(ByVal Value As Boolean)
    p_transparent = Value
  End Set
  End Property

  Public Sub UpdateFrame()
    Static cubeangle As Single
    Static lastFrameUpdate As Integer

    Try
      cubeangle += CSng((Environment.TickCount - lastFrameUpdate) / 1000)
      If cubeangle >= Math.PI * 2 Then cubeangle = 0
      p_D3DDevice.Transform.World = Matrix.Multiply(Matrix.RotationY(cubeangle), Matrix.RotationX(cubeangle))

      If (Environment.TickCount() - p_LastFPSCheck >= 1000) Then
        p_LastFPSCheck = Environment.TickCount()
        p_FPS = p_FrameCount
        Console.WriteLine(p_FPS)
        p_FrameCount = 0
      End If
      p_FrameCount += 1
    Catch
    Finally
      lastFrameUpdate = Environment.TickCount
    End Try
  End Sub

  Public Sub RenderFrame()
    Try
      If Not p_Init Then Return
      With p_D3DDevice
        .Clear(ClearFlags.ZBuffer Or ClearFlags.Target, Color.Black, 1, 0)
        .BeginScene()

        .SetStreamSource(0, p_vbCube, 0)
        If p_transparent Then
          .SetTexture(0, p_TransparentCubeTexture)
        Else
          .SetTexture(0, p_CubeTexture)
        End If
        .VertexFormat = CustomVertex.PositionTextured.Format
        .DrawPrimitives(PrimitiveType.TriangleList, 0, 12)

        p_Font.DrawText("FPS: " & p_FPS & "   Transparenz (t): " & p_transparent, _
          New Rectangle(10, 10, 400, 50), DrawTextFormat.None, Color.Red)

        .EndScene()
        .Present()
      End With
    Catch
    End Try
  End Sub
End Class
